unit RGS_DSChooser;

{ TRgsDSChooser Version 2.0
  Author : Tnis Argus
  Status: Freeware
  eMail : Tonis.Argus@mail.ee

  This component enables You to choose a dataset and then
  view it vertically
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, DB,
  dsgnintf, extctrls, stdctrls, RGS_DBInspector, dbctrls,
  RGS_FieldNotes, RGS_FieldStore;

type
  TDSChooserOptions = set of (coFlat);
  TReadOnlyState = (roDefault,roNever,roAlways);

  TRgsDSChooserChangeEvent = procedure(Sender: TObject; Datasource : TDatasource) of object;
  TRgsDSChooserDBNavClick = procedure(Sender: TObject; Button: TNavigateBtn) of object;
  TRgsDSChooserDBNavBeforeAction = procedure(Sender: TObject; Button: TNavigateBtn) of object;

  TRgsDSChooserItem = class(TCollectionItem)
  private
    FDatasource: TDatasource;
    FDatasourceName : String;
    FShowNavigator : Boolean;
    FVisibleButtons: TButtonSet;
    FReadOnlyState : TReadOnlyState;
  protected
    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Datasource: TDatasource read FDatasource write FDatasource;
    property DatasourceName : String read FDatasourceName write FDatasourceName;
    property ShowNavigator : Boolean read FShowNavigator write FShowNavigator;
    property VisibleButtons: TButtonSet read FVisibleButtons write FVisibleButtons
      default [nbFirst, nbPrior, nbNext, nbLast, nbInsert, nbDelete, nbRefresh];
    property ReadOnlyState : TReadOnlyState read FReadOnlyState write FReadOnlyState;
  end;

  TRgsDSChooserCollection = class(TCollection)
  protected
    FOwner: TPersistent;
    function GetOwner:TPersistent; override;
    function GetItem(Index: Integer): TRGSDSChooserItem;
    procedure SetItem(Index: Integer; Value: TRGSDSChooserItem);
  public
    property Items[Index: Integer]: TRGSDSChooserItem read GetItem write SetItem; default;
  end;

  TRgsDSChooser = class(TCustomPanel)
  private
    FRGSDBInspector: TRGSDBInspector;
    FOLDVisibleButtons: TButtonSet;
    FOnDatasourceChange: TRgsDSChooserChangeEvent;
    FRgsDSChooserDBNavClick : TRgsDSChooserDBNavClick;
    FRgsDSChooserDBNavBeforeAction : TRgsDSChooserDBNavBeforeAction;
    FDSCollection: TRGSDSChooserCollection;
    FRGSFieldNotes : TRGSFieldNotes;
    FFieldNotes : Boolean;
    FTopPanel: TPanel;
    FComboBox: TComboBox;
    FDBNavigator : TDBNavigator;
    FTextMemo : String;
    FTextImage : String;
    FLoaded : Boolean;
    FReadOnly : Boolean;
    FDSChooserOptions : TDSChooserOptions;
    procedure SetInspector(ItemIndex : Integer);
    function GetIndexByDS(aDataSource : TDataSource) : Integer;
    procedure SetReadOnly(Value : Boolean);
    procedure TopPanelResize(Sender: TObject);
    procedure ComboBoxChange(Sender: TObject);
    procedure rgsDBNavBeforeAction(Sender: TObject; Button: TNavigateBtn);
    procedure rgsDBNavClick(Sender: TObject; Button: TNavigateBtn);
    procedure SetRGSFieldNotes(Value : TRGSFieldNotes);
    function IsChooserDataSource(aDataSource : TDataSource) : Boolean;
  protected
    procedure SetCollection(const Value: TRGSDSChooserCollection);
    procedure Loaded; override;
    procedure Repaint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetDataSource(aDataSource : TDataSource);
    procedure ResetInspector;
    procedure AddFieldDefs(aRGSFieldStore : TRGSFieldStore; mode : TRgsFieldsAddMode);
  published
    property Datasources: TRGSDSChooserCollection read FDSCollection write SetCollection;
    property ReadOnly : boolean read FReadOnly write SetReadOnly;
    property TextMemo : String read FTextMemo write FTextMemo;
    property TextImage : string read FTextImage write FTextImage;
    property DSChooserOptions : TDSChooserOptions read FDSChooserOptions write FDSChooserOptions;
    property RGSFieldNotes : TRGSFieldNotes read FRGSFieldNotes write SetRGSFieldNotes;
    property FieldNotes : Boolean read FFieldNotes write FFieldNotes;
    property Align;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property Enabled;
    property Visible;
    property TabOrder;
    property OnResize;
    property OnDatasourceChange : TRgsDSChooserChangeEvent
             read FOnDatasourceChange write FOnDatasourceChange;
    property DBNavBeforeAction : TRgsDSChooserDBNavBeforeAction
             read FRgsDSChooserDBNavBeforeAction write FRgsDSChooserDBNavBeforeAction;
    property OnDBNavClick : TRgsDSChooserDBNavClick
             read FRgsDSChooserDBNavClick write FRgsDSChooserDBNavClick;
  end;

  TRgsDSChooserEditor = class(TComponentEditor)
    function GetVerbCount: integer; override;
    function GetVerb(Index: integer): string; override;
    procedure ExecuteVerb(Index: integer); override;
  end;

procedure Register;

implementation

constructor TRgsDSChooserItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FShowNavigator := False;
  FVisibleButtons := [nbFirst,nbPrior,nbNext,nbLast,nbInsert,nbDelete,nbRefresh];
end;

destructor TRgsDSChooserItem.Destroy;
begin
   inherited;
end;

procedure TRgsDSChooserItem.Assign(Source: TPersistent);
begin
   if Source is TRGSDSChooserItem then
      begin
        Datasource := TRGSDSChooserItem(Source).Datasource;
        DatasourceName := TRGSDSChooserItem(Source).DatasourceName;
        ShowNavigator := TRGSDSChooserItem(Source).ShowNavigator;
        VisibleButtons := TRGSDSChooserItem(Source).VisibleButtons;
        ReadOnlyState := TRGSDSChooserItem(Source).ReadOnlyState;
      end
   else
     inherited Assign(Source);
end;

function TRgsDSChooserItem.GetDisplayName: string;
begin
   Result := DatasourceName;
   if Result = '' then
     Result := inherited GetDisplayName;
end;

function TRgsDSChooserCollection.GetOwner: TPersistent;
begin
   Result := FOwner;
end;

function TRgsDSChooserCollection.GetItem(Index: Integer): TRGSDSChooserItem;
begin
   Result := TRGSDSChooserItem(inherited GetItem(Index));
end;

procedure TRgsDSChooserCollection.SetItem(Index: Integer; Value: TRGSDSChooserItem);
begin
   inherited SetItem(Index, Value);
end;

constructor TRgsDSChooser.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Height := 200;
  Width := 160;
  FDSCollection := TRGSDSChooserCollection.Create(TRGSDSChooserItem);
  FDSCollection.FOwner := Self;

  FTopPanel:= TPanel.Create(Self);
  FTopPanel.Parent := Self;
  FTopPanel.Align := alTop;
  FTopPanel.Height := 23;
  FTopPanel.OnResize := TopPanelResize;

  FComboBox:= TComboBox.Create(Self);
  FComboBox.Parent := FTopPanel;
  FComboBox.Top := 0;
  FComboBox.Left := 0;
  FComboBox.Width := FTopPanel.Width-3;
  FComboBox.OnChange := ComboBoxChange;

  FDBNavigator := TDBNavigator.Create(Self);
  FDBNavigator.Parent := Self;
  FDBNavigator.Align := alTop;
  FDBNavigator.Height := FDBNavigator.Height-5;
  FDBNavigator.Visible := False;
//  FDBNavigator.Flat := True;
  FDBNavigator.VisibleButtons := [nbFirst,nbPrior,nbNext,nbLast,nbInsert,nbDelete,nbRefresh];
  FDBNavigator.BeforeAction := rgsDBNavBeforeAction;
  FDBNavigator.OnClick := rgsDBNavClick;

  FRGSDBInspector:= TRGSDBInspector.Create(Self);
  FRGSDBInspector.Parent := Self;
  FRGSDBInspector.Align := alClient;

  FLoaded := False;
  FFieldNotes := False;
end;

destructor TRgsDSChooser.Destroy;
begin
   FDSCollection.Free;
   inherited Destroy;
end;

procedure TRgsDSChooser.SetReadOnly(Value : Boolean);
begin
  FReadOnly := Value;
  If FComboBox.ItemIndex = -1 Then
    exit;
  If FDSCollection.Items[FComboBox.ItemIndex].ReadOnlyState = roNever Then
    FRGSDBInspector.ReadOnly := False
  else
    If FDSCollection.Items[FComboBox.ItemIndex].ReadOnlyState = roAlways Then
      FRGSDBInspector.ReadOnly := True
    else
      FRGSDBInspector.ReadOnly := Value;
  If FRGSDBInspector.ReadOnly Then
    FDBNavigator.VisibleButtons := FDBNavigator.VisibleButtons - [nbEdit,nbPost,nbCancel,nbInsert,nbDelete]
  else
    Begin
      If nbEdit in FOLDVisibleButtons Then
        FDBNavigator.VisibleButtons := FDBNavigator.VisibleButtons + [nbEdit];
      If nbPost in FOLDVisibleButtons Then
        FDBNavigator.VisibleButtons := FDBNavigator.VisibleButtons + [nbPost];
      If nbCancel in FOLDVisibleButtons Then
        FDBNavigator.VisibleButtons := FDBNavigator.VisibleButtons + [nbCancel];
      If nbInsert in FOLDVisibleButtons Then
        FDBNavigator.VisibleButtons := FDBNavigator.VisibleButtons + [nbInsert];
      If nbDelete in FOLDVisibleButtons Then
        FDBNavigator.VisibleButtons := FDBNavigator.VisibleButtons + [nbDelete];
    end;
end;

procedure TRgsDSChooser.TopPanelResize(Sender: TObject);
begin
  FComboBox.Width := FTopPanel.ClientWidth;
end;

procedure TRgsDSChooser.rgsDBNavBeforeAction(Sender: TObject; Button: TNavigateBtn);
begin
  If Assigned(FRgsDSChooserDBNavBeforeAction) Then
    FRgsDSChooserDBNavBeforeAction(Sender, Button);
end;

procedure TRgsDSChooser.rgsDBNavClick(Sender: TObject; Button: TNavigateBtn);
begin
  If Assigned(FRgsDSChooserDBNavClick) Then
    FRgsDSChooserDBNavClick(Sender, Button);
end;

procedure TRgsDSChooser.ComboBoxChange(Sender: TObject);
begin
  SetInspector(FComboBox.ItemIndex);
end;

procedure TRgsDSChooser.SetInspector(ItemIndex : Integer);
begin
  If (FDSCollection.Items[ItemIndex].DataSource <> nil) And
     (FDSCollection.Items[ItemIndex].DataSource.DataSet <> nil) Then
    begin
      FRGSDBInspector.DataSource := FDSCollection.Items[ItemIndex].Datasource;
      FRGSDBInspector.FieldNotes := FFieldNotes;
      If FDSCollection.Items[ItemIndex].ReadOnlyState = roNever Then
        FRGSDBInspector.ReadOnly := False
      else
        If FDSCollection.Items[ItemIndex].ReadOnlyState = roAlways Then
          FRGSDBInspector.ReadOnly := True
        else
          FRGSDBInspector.ReadOnly := FReadOnly;
      FDBNavigator.Visible := False;
      FDBNavigator.DataSource := nil;
      if FDSCollection.Items[ItemIndex].FShowNavigator Then
        begin
          FDBNavigator.DataSource := FDSCollection.Items[ItemIndex].Datasource;
          FDBNavigator.VisibleButtons := FDSCollection.Items[ItemIndex].VisibleButtons;
          FOLDVisibleButtons := FDBNavigator.VisibleButtons;
          If FRGSDBInspector.ReadOnly Then
            FDBNavigator.VisibleButtons := FDBNavigator.VisibleButtons - [nbEdit,nbPost,nbCancel,nbInsert,nbDelete];
          FDBNavigator.Visible := True;
          FTopPanel.Top := 0;
        end;
      if Assigned(FOnDatasourceChange) then
        FOnDatasourceChange(Self, FRGSDBInspector.DataSource);
    end;
end;

procedure TRgsDSChooser.SetDataSource(aDataSource : TDataSource);
var
  newIndex : Integer;
begin
  newIndex := GetIndexByDS(aDataSource);
  If newIndex = -1 Then
    exit;
  SetInspector(newIndex);
  FComboBox.ItemIndex := newIndex;
end;

procedure TRgsDSChooser.ResetInspector;
begin
  SetInspector(FComboBox.ItemIndex);
end;

function TRgsDSChooser.GetIndexByDS(aDataSource : TDataSource) : Integer;
var
  i : integer;
begin
  result := -1;
  for i := 0 to FDSCollection.Count-1 do
    If aDataSource = FDSCollection.Items[i].Datasource Then
      begin
        result := i;
        break;
      end;
end;

procedure TRgsDSChooser.SetCollection(const Value: TRGSDSChooserCollection);
begin
   FDSCollection.Assign(Value);
end;

procedure TRgsDSChooser.Loaded;
Var
  i : Integer;
begin
  inherited;
  FRGSDBInspector.TextMemo := TextMemo;
  FRGSDBInspector.TextImage := TextImage;
  if (csDesigning in ComponentState) then
    FDBNavigator.Visible := False;
  If coFlat in DSChooserOptions Then
    FDBNavigator.Flat := True;
  If Not FLoaded Then
    For i := 0 To FDSCollection.Count-1 do
      FComboBox.Items.Add(FDSCollection.Items[i].DatasourceName);
  FLoaded := True;
  If FDSCollection.Count > 0 Then begin
    SetInspector(0);
    FComboBox.ItemIndex := 0;
  end;
end;

procedure TRgsDSChooser.Repaint;
begin
  inherited;
  FRGSDBInspector.Repaint;
end;

procedure TRgsDSChooser.SetRGSFieldNotes(Value : TRGSFieldNotes);
begin
  FRGSFieldNotes := Value;
  FRGSDBInspector.RGSFieldNotes := Value;
  FFieldNotes := (Value <> nil);
end;

function TRgsDSChooser.IsChooserDataSource(aDataSource : TDataSource) : Boolean;
var
  i : integer;
begin
  result := False;
  for i := 0 to FDSCollection.Count-1 do
    If FDSCollection[i].Datasource = aDataSource Then
      Begin
        result := True;
        exit;
      end;
end;

procedure TRgsDSChooser.AddFieldDefs(aRGSFieldStore : TRGSFieldStore; mode : TRgsFieldsAddMode);
var
  i : integer;
begin
  If mode = faClearExisting Then
    FRGSDBInspector.ClearFieldDefs;
  for i := 0 to aRGSFieldStore.FieldDefs.Count-1 do
    If IsChooserDataSource(aRGSFieldStore.FieldDefs[i].DataSource) Then
      FRGSDBInspector.AddFieldDefItem(aRGSFieldStore.FieldDefs[i]);
end;

function TRgsDSChooserEditor.GetVerbCount: integer;
begin
   Result := 1;
end;

function TRgsDSChooserEditor.GetVerb(Index: integer): string;
begin
   Result := 'Add/Remove datasources...';
end;

procedure TRgsDSChooserEditor.ExecuteVerb(Index: integer);
begin
  Designer.Modified;
end;


procedure Register;
begin
  RegisterComponents('Freeware', [TRgsDSChooser]);
  RegisterComponentEditor(TRgsDSChooser, TRgsDSChooserEditor);
end;

end.
